package rap.virtualtree;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.viewers.ILazyTreeContentProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.rap.rwt.lifecycle.IEntryPoint;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Tree;

public class VirtualTreeTest implements IEntryPoint {
	public int createUI() {
		Display display = new Display();
		Shell shell = new Shell(display);
		shell.setLayout(new GridLayout(1, false));

		Tree tree = new Tree(shell, SWT.VIRTUAL);
		tree.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		final TreeViewer treeViewer = new TreeViewer(tree);
		treeViewer.setUseHashlookup(true);

		final Map<String, String[]> parent2Children = new HashMap<String, String[]>();
		parent2Children.put("root", new String[] { "1", "2", "3" });
		parent2Children.put("1", new String[] { "1.1", "1.2" });
		parent2Children.put("2", new String[] { "2.1", "2.2" });
		parent2Children.put("3", new String[] { "3.1", "3.2" });

		treeViewer.setContentProvider(new ILazyTreeContentProvider() {
			@Override
			public void inputChanged(Viewer viewer, Object oldInput,
					Object newInput) {
			}

			@Override
			public void dispose() {
			}

			@Override
			public void updateElement(Object parent, int index) {
				if (parent != null) {
					Object[] children = parent2Children.get(parent);
					if (index < children.length) {
						treeViewer.replace(parent, index, children[index]);
						updateChildCount(children[index], -1);
					}
				}
			}

			@Override
			public void updateChildCount(Object element, int currentChildCount) {
				Object[] children = parent2Children.get(element);
				treeViewer.setChildCount(element, children == null ? 0
						: children.length);
			}

			@Override
			public Object getParent(Object element) {
				return null;
			}
		});
		treeViewer.setInput("root");

		Button button = new Button(shell, SWT.PUSH);
		button.setText("Test");
		button.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				parent2Children.put("root", new String[] { "1", "3" });
				parent2Children.put("1", new String[] { "1.1", "1.2", "2" });
				treeViewer.refresh();
			}
		});
		shell.open();
		return 0;
	}
}